/* --COPYRIGHT--,BSD
 * Copyright (c) 2014, Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * --/COPYRIGHT--*/
/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package SMBUsDemo;

import javax.swing.ImageIcon;

/**
 *
 * @author a0272600
 */
/*!
* \class DIM430_Comm
*
* \brief Class used to handle DIM430 Slave
*
* The DIM430 slave is a slave dimmer. This class has all functions needed to 
*  send commands to the Master device in order to read/write registers of the 
*  DIM430. Responses from DIM430->Master->PC will be reflected in the corresponding
*  GUI components. 
*  
*/
public class DIM430_Comm extends Slave_Comm{
    CommThread commthread;   
    public static final char READ_DIM430_DEVICE_ID = 'p';
    public static final char WRITE_DIM430_CONFIG = 'Y';
    public static final char READ_DIM430_CONFIG = 'y';
    public static final char WRITE_DIM430_DUTY = 'U';
    public static final char READ_DIM430_DUTY = 'u';

    public static final char RESPONSE_DIM430_DEVID = 0x08;
    public static final char RESPONSE_DIM430_CONFIG = 0x09;
    public static final char RESPONSE_DIM430_DUTY = 0x0A;
    
    public DIM430_Comm() {
         commthread = new CommThread(); 
         commthread.start();
     }
    
    @Override
    public synchronized void Disable() {
         is_enabled = false;
         is_connected = false;
        GUI.Disable_DIM430_GUIComponents();
        GUI.Icon_DIM430.setIcon(new ImageIcon(getClass().getResource("/Resources/red.png")));
     }
     
    public void Change_Config(){
         byte[] data = new byte[] {
            0x00,                   // LSB
            0x00};                  // MSB
        
         if (GUI.jCheckBox_DIM430_OE.isSelected() == true)
            data[0] |= 0x01;
         
        Write_Config(data);
     }
    
    public void Change_Duty(){
         byte[] data = new byte[] {
            0x00,                   // LSB
            0x00};                  // MSB
        
         // Set the duty value from 0 to 100
        data[0] = (byte) GUI.jSlider_DIM430_Duty.getValue();
        if (data[0] >100)
            data[0] = 100;
        
        Write_Duty(data);
     }
    
    @Override
    public Boolean ProcessResponse()
    {
        String CmdResp;
        byte[] CmdRespByte;
        GUI.CommInterface Comm = GUI.MyCommSM;
        switch (Comm.RX_Cmd)
        {
            case RESPONSE_DIM430_DEVID:
                CmdResp  = this.Get_ResponseAsString();
                if (CmdResp != null)
                {
                    GUI.Text_DIM430_DEVICEID.setText(CmdResp); 
                    if (Comm.TX_Cmd ==  READ_DIM430_DEVICE_ID)
                    {
                        synchronized(stick_WaitCommand)
                        {
                            command_found = true;
                            stick_WaitCommand.notify();
                        }
                    }

                    Comm.SM = GUI.Response_SM.SM_DONE;  // Packet reception is complete
                }
            break;
               
            case RESPONSE_DIM430_CONFIG:
               CmdRespByte  = this.Get_ResponseAsArray();
                if (CmdRespByte != null)
                {
                    if ((CmdRespByte[0] & 0x01) == 0x01)
                            GUI.jCheckBox_DIM430_OE.setSelected(true);
                        else
                            GUI.jCheckBox_DIM430_OE.setSelected(false);

                        if (Comm.TX_Cmd ==  READ_DIM430_CONFIG)
                        {
                            synchronized(stick_WaitCommand)
                            {
                                command_found = true;
                                stick_WaitCommand.notify();
                            }
                        }

                        Comm.SM = GUI.Response_SM.SM_DONE;  // Packet reception is complete
                }
            break;
                
            case RESPONSE_DIM430_DUTY:
               CmdRespByte  = this.Get_ResponseAsArray();
                if (CmdRespByte != null)
                {
                    GUI.jSlider_DIM430_Duty.setValue(CmdRespByte[0] + CmdRespByte[1]<<8);

                    if (Comm.TX_Cmd ==  READ_DIM430_DUTY)
                    {
                        synchronized(stick_WaitCommand)
                        {
                            command_found = true;
                            stick_WaitCommand.notify();
                        }
                    }

                    Comm.SM = GUI.Response_SM.SM_DONE;  // Packet reception is complete
                }
            break;
            default:
                System.out.println("Incorrect");       
                break;
        }
        
        return true;
    }
    
    public void Read_DeviceID()
    {
        byte[] command = new byte[] {
            (byte)0x80,     // Header
            0x01,           // Len
            READ_DIM430_DEVICE_ID};     // Command

        SendCommand(command);
    }
    
    public Boolean Read_DeviceID_WaitUntilDone()
    {
        byte[] command = new byte[] {
            (byte)0x80,     // Header
            0x01,           // Len
            READ_DIM430_DEVICE_ID};     // Command

        try {
            return SendCommand_WaitUntilDone(command);
        }
        catch (InterruptedException e1){
                e1.printStackTrace();
                return false;
            }
    }
    
    public void Read_Config()
    {
        byte[] command = new byte[] {
            (byte)0x80,     // Header
            0x01,           // Len
            READ_DIM430_CONFIG};     // Command

        SendCommand(command);
    }
    
    public Boolean Read_Config_WaitUntilDone()
    {
        byte[] command = new byte[] {
            (byte)0x80,     // Header
            0x01,           // Len
            READ_DIM430_CONFIG};     // Command

        try {
            return SendCommand_WaitUntilDone(command);
        }
        catch (InterruptedException e1){
                e1.printStackTrace();
                return false;
            }
    }
    
    public void Write_Config( byte[] data)
    {
        byte[] command = new byte[] {
            (byte)0x80,     // Header
            0x03,           // Len
            WRITE_DIM430_CONFIG,    // Command
            data[0],           // Data L
            data[1]};          // Data H

        SendCommand(command);
    }
    
    public void Read_Duy()
    {
        byte[] command = new byte[] {
            (byte)0x80,     // Header
            0x01,           // Len
            READ_DIM430_DUTY};     // Command

        SendCommand(command);
    }
    
    public Boolean Read_Duty_WaitUntilDone()
    {
        byte[] command = new byte[] {
            (byte)0x80,     // Header
            0x01,           // Len
            READ_DIM430_CONFIG};     // Command

        try {
            return SendCommand_WaitUntilDone(command);
        }
        catch (InterruptedException e1){
                e1.printStackTrace();
                return false;
            }
    }
    
    public void Write_Duty( byte[] data)
    {
        byte[] command = new byte[] {
            (byte)0x80,     // Header
            0x03,           // Len
            WRITE_DIM430_DUTY,    // Command
            data[0],           // Data L
            data[1]};          // Data H

        SendCommand(command);
    }
    
     public class CommThread extends Thread {
        @Override
        public void run(){
            try {
                while (true) {
                    while (is_enabled == false)
                    {
                        synchronized (stick) {
                            stick.wait();
                          }
                    }
                    if (is_enabled == true)
                    {
                        // If not connected yet, try to connect for first time
                        if (is_connected == false)
                        {
                            is_connected = true; // enable for now
                            if ((Read_DeviceID_WaitUntilDone() == true) && 
                                (Read_Config_WaitUntilDone() == true))
                            {
                                GUI.Enable_DIM430_GUIComponents();
                                is_connected = true;
                                GUI.Icon_DIM430.setIcon(new ImageIcon(getClass().getResource("/Resources/green.png")));
                            }
                            else
                            {
                                GUI.Disable_DIM430_GUIComponents();
                                GUI.Icon_DIM430.setIcon(new ImageIcon(getClass().getResource("/Resources/red.png")));
                                is_connected = false;
                            }
                            CommThread.sleep(500);
                        }
                        else
                        {
                            // If already connected yet, check if communication is still OK periodically
                            if (Read_DeviceID_WaitUntilDone() == true)
                            {
                                is_connected = true;
                                GUI.Enable_DIM430_GUIComponents();
                                GUI.Icon_DIM430.setIcon(new ImageIcon(getClass().getResource("/Resources/green.png")));
                            }
                            else
                            {
                                GUI.Disable_DIM430_GUIComponents();
                                GUI.Icon_DIM430.setIcon(new ImageIcon(getClass().getResource("/Resources/red.png")));
                                is_connected = false;
                            }
                            CommThread.sleep(2000);
                        }
                    }
                }
            } catch (InterruptedException e) {
                e.printStackTrace();
            }
        }
     }
}
