/* --COPYRIGHT--,BSD
 * Copyright (c) 2014, Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * --/COPYRIGHT--*/
/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package SMBUsDemo;


import java.awt.Color;
import java.util.concurrent.Semaphore;
import javax.swing.ImageIcon;
import org.jfree.chart.ChartPanel;
import org.jfree.chart.plot.MeterPlot;

import SMBUsDemo.*;
import static SMBUsDemo.GUI.MyCommSM;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.concurrent.Callable;
import java.util.concurrent.Executors;
import java.util.concurrent.ScheduledExecutorService;
import java.util.concurrent.ScheduledFuture;
import java.util.concurrent.TimeUnit;
import javax.swing.JLabel;
import javax.swing.text.BadLocationException;


/**
 *
 * @author a0272600
 */

/*!
 * \class GUI
 *
 * \brief Class handling GUI
 *
 * This is the main class of the application, initializing other classes and
 *  handling user interaction
 *  
 */
public class GUI extends javax.swing.JPanel {

    static MyThermometer AmbientTherm;
    static MyThermometer ObjectTherm;
    static TMP006_Comm   TMP006_comm = new TMP006_Comm();
    static DIM430_Comm   DIM430_comm = new DIM430_Comm();
    
    /*!
    * \class CommInterface
    *
    * \brief Handles synchronization with the Communication interface
    *  
    */
    static class CommInterface
    {
        public CommInterface()
        {
            TX_Cmd = 0;
            RX_Counter = 0;
            WaitCmd = false;
            SM = GUI.Response_SM.SM_WAIT_HDR;
            RX_Len = 0;
            RX_Cmd = 0;
            SMB_Resp_Buff = new byte[33];
            mutex = new Semaphore(1);
            SMB_TX_Len = 0;
            SMB_TX_Buff = new byte[33];
        }
        GUI.Response_SM SM;
        int TX_Cmd;
        Boolean WaitCmd;
        int RX_Counter;
        int RX_Len;
        int RX_Cmd = 0;
        byte[] SMB_Resp_Buff;
        Semaphore mutex ;
        Communicator communicator;
        int SMB_TX_Len;
        byte[] SMB_TX_Buff;
    }
    //Communicator object
    public static CommInterface MyCommSM;
    
    public static BlockDiagram_Block Block_PC;
    public static BlockDiagram_Block Block_Master;
    public static BlockDiagram_Block Block_PCtoMaster;
    public static BlockDiagram_Block Block_MastertoPC;
    public static BlockDiagram_Block Block_SMbus;
    public static BlockDiagram_Block Block_Slave;
    public static BlockDiagram_Block Block_SlavetoSMB;
    public static BlockDiagram_Block Block_SMBtoSlave;
    public static BlockDiagram_Block Block_TMP006toSMB;
    public static BlockDiagram_Block Block_SMBtoTMP006;
    public static BlockDiagram_Block Block_TMP006;
    
    /*!
    * \class BlockDiagram_Block
    *
    * \brief Controls the block diagram pieces
    *
    * Allows to turn some blocks "ON" or "OFF" by choosing the appropiate 
    *  image
    *  
    */
    public static class BlockDiagram_Block
    {
        java.lang.Object stick_Timeout;
        Boolean Timeout = false;
        String On_Pic;
        String Off_Pic;
        JLabel Label;
        ScheduledExecutorService scheduledExecutorService = Executors.newScheduledThreadPool(1);
        
        public BlockDiagram_Block( String On, String Off, JLabel l)
        {
            stick_Timeout = new java.lang.Object();
            Timeout = false;
            On_Pic = On;
            Off_Pic = Off;
            Label = l;
        }
        
        public void Blink(int timeoutms)
        {
            Label.setIcon(new ImageIcon(getClass().getResource(On_Pic)));
            ScheduledFuture scheduledFuture = scheduledExecutorService.schedule(new Callable() {
                @Override
                public Object call() throws Exception {
                    synchronized(stick_Timeout)
                    {
                        TurnOff();
                        stick_Timeout.notify();
                    }
                    //System.out.println("OK");
                    return "Called!";
                }
            },timeoutms, TimeUnit.MILLISECONDS);
        }
        
        public void TurnOn()
        {
            Label.setIcon(new ImageIcon(getClass().getResource(On_Pic)));
        }

        public void TurnOff()
        {
            Label.setIcon(new ImageIcon(getClass().getResource(Off_Pic)));
        }

    }
    
    /**
     * Creates new form GUI
     */
    public GUI() {
        AmbientTherm = new MyThermometer();
        AmbientTherm.SetTitle("Ambient Temperature");
        AmbientTherm.r = 0;
        AmbientTherm.g = 255;   // SHow color as Green
        AmbientTherm.b = 0;
        AmbientTherm.SetRanges(AmbientTherm.GetMinTemp(), AmbientTherm.GetMaxTemp());
        ObjectTherm = new MyThermometer();
        ObjectTherm.SetTitle("Object Temperature");
        ObjectTherm.r = 255;   // SHow color as Red
        ObjectTherm.g = 0;   
        ObjectTherm.b = 0;
        ObjectTherm.SetRanges(ObjectTherm.GetMinTemp(), ObjectTherm.GetMaxTemp());
        initComponents();
        createObjects();
        MyCommSM.communicator.searchForPorts();
        Block_PC = new BlockDiagram_Block("/Resources/PC_On.png", 
                                          "/Resources/PC_Off.png", 
                                            Image_BD_PC);
        Block_Master = new BlockDiagram_Block("/Resources/Master_On.png", 
                                          "/Resources/Master_Off.png", 
                                            Image_BD_Master);
        Block_PCtoMaster = new BlockDiagram_Block("/Resources/PC_to_Master_On.png", 
                                          "/Resources/PC_to_Master_Off.png", 
                                            Image_BD_PCtoMaster);
        Block_MastertoPC= new BlockDiagram_Block("/Resources/Master_to_PC_On.png", 
                                          "/Resources/Master_to_PC_Off.png", 
                                            Image_BD_MastertoPC);
        Block_SMbus= new BlockDiagram_Block("/Resources/SMBus_On.png", 
                                          "/Resources/SMBus_Off.png", 
                                            Image_BD_SMBus);
        Block_Slave= new BlockDiagram_Block("/Resources/Slave_On.png", 
                                          "/Resources/Slave_Off.png", 
                                            Image_BD_Slave);
        Block_TMP006= new BlockDiagram_Block("/Resources/TMP006_On.png", 
                                          "/Resources/TMP006_Off.png", 
                                            Image_BD_TMP006);
        Block_SlavetoSMB= new BlockDiagram_Block("/Resources/Up_On.png", 
                                          "/Resources/Up_Off.png", 
                                            Image_BD_SlavetoHost);
        Block_SMBtoSlave= new BlockDiagram_Block("/Resources/Down_On.png", 
                                          "/Resources/Down_Off.png", 
                                            Image_BD_HosttoSlave);
        Block_TMP006toSMB= new BlockDiagram_Block("/Resources/Up_On.png", 
                                          "/Resources/Up_Off.png", 
                                            Image_BD_TMP006toHost);
        Block_SMBtoTMP006= new BlockDiagram_Block("/Resources/Down_On.png", 
                                          "/Resources/Down_Off.png", 
                                            Image_BD_HosttoTMP006);
    }
    
           
    private void createObjects()
    {
        MyCommSM = new CommInterface();
        MyCommSM.communicator = new Communicator(this);
    }

    /**
     * This method is called from within the constructor to initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is always
     * regenerated by the Form Editor.
     */
    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        jLabel12 = new javax.swing.JLabel();
        jPanelTMP006 = new javax.swing.JPanel();
        jLabel3 = new javax.swing.JLabel();
        Text_TMP006_MANUFID = new javax.swing.JTextField();
        jLabel4 = new javax.swing.JLabel();
        Text_TMP006_DEVICEID = new javax.swing.JTextField();
        jLabel2 = new javax.swing.JLabel();
        jComboBox_TMP006_ConvRate = new javax.swing.JComboBox();
        jLabel6 = new javax.swing.JLabel();
        jComboBox_TMP006_MOD = new javax.swing.JComboBox();
        jCheckBox_TMP006_DRDY = new javax.swing.JCheckBox();
        Button_TMP006_ReadV = new javax.swing.JButton();
        Text_TMP006_VHex = new javax.swing.JTextField();
        Text_TMP006_THex = new javax.swing.JTextField();
        Button_TMP006_ReadT = new javax.swing.JButton();
        Text_TMP006_TC = new javax.swing.JTextField();
        Text_TMP006_Volt = new javax.swing.JTextField();
        Icon_TMP006 = new javax.swing.JLabel();
        jPanelComm = new javax.swing.JPanel();
        jLabel5 = new javax.swing.JLabel();
        cboxPorts = new javax.swing.JComboBox();
        btnConnect = new javax.swing.JButton();
        btnDisconnect = new javax.swing.JButton();
        Icon_Comm = new javax.swing.JLabel();
        jPanelSMB = new javax.swing.JPanel();
        Icon_Status = new javax.swing.JLabel();
        jLabel7 = new javax.swing.JLabel();
        jScrollPane1 = new javax.swing.JScrollPane();
        jTextArea_SMBLog = new javax.swing.JTextArea();
        jCheckBox_SMBLog = new javax.swing.JCheckBox();
        jButton_ClearSMBLog = new javax.swing.JButton();
        jPanel_AmbienTemp = AmbientTherm.chartPanel;
        jPanel_ObjectTemp = ObjectTherm.chartPanel;
        jPanel3 = new javax.swing.JPanel();
        jLabel1 = new javax.swing.JLabel();
        jPanel4 = new javax.swing.JPanel();
        jLabel8 = new javax.swing.JLabel();
        Text_DIM430_DEVICEID = new javax.swing.JTextField();
        jCheckBox_DIM430_OE = new javax.swing.JCheckBox();
        jSlider_DIM430_Duty = new javax.swing.JSlider();
        jLabel_DIM430Duty = new javax.swing.JLabel();
        Icon_DIM430 = new javax.swing.JLabel();
        jPanel_BlockDiagram = new javax.swing.JPanel();
        Image_BD_PC = new javax.swing.JLabel();
        Image_BD_PCtoMaster = new javax.swing.JLabel();
        Image_BD_MastertoPC = new javax.swing.JLabel();
        Image_BD_Master = new javax.swing.JLabel();
        Image_BD_SMBus = new javax.swing.JLabel();
        Image_BD_HosttoSlave = new javax.swing.JLabel();
        Image_BD_Slave = new javax.swing.JLabel();
        Image_BD_SlavetoHost = new javax.swing.JLabel();
        Image_BD_TMP006 = new javax.swing.JLabel();
        Image_BD_TMP006toHost = new javax.swing.JLabel();
        Image_BD_HosttoTMP006 = new javax.swing.JLabel();

        setAutoscrolls(true);

        jLabel12.setIcon(new javax.swing.ImageIcon(getClass().getResource("/Resources/TI_platform_bar_red.png"))); // NOI18N

        jPanelTMP006.setBorder(javax.swing.BorderFactory.createBevelBorder(javax.swing.border.BevelBorder.RAISED));

        jLabel3.setFont(new java.awt.Font("Arial", 0, 10)); // NOI18N
        jLabel3.setText("Manufacturer ID");

        Text_TMP006_MANUFID.setEditable(false);
        Text_TMP006_MANUFID.setFont(new java.awt.Font("Arial", 0, 10)); // NOI18N
        Text_TMP006_MANUFID.setEnabled(false);

        jLabel4.setFont(new java.awt.Font("Arial", 0, 10)); // NOI18N
        jLabel4.setText("Device ID");

        Text_TMP006_DEVICEID.setEditable(false);
        Text_TMP006_DEVICEID.setFont(new java.awt.Font("Arial", 0, 10)); // NOI18N
        Text_TMP006_DEVICEID.setEnabled(false);

        jLabel2.setFont(new java.awt.Font("Arial", 0, 10)); // NOI18N
        jLabel2.setText("Conversion Rate");

        jComboBox_TMP006_ConvRate.setFont(new java.awt.Font("Arial", 0, 10)); // NOI18N
        jComboBox_TMP006_ConvRate.setModel(new javax.swing.DefaultComboBoxModel(new String[] { "4 conv/sec", "2 conv/sec", "1 conv/sec", "0.5 conv/sec", "0.25 conv/sec" }));
        jComboBox_TMP006_ConvRate.setEnabled(false);
        jComboBox_TMP006_ConvRate.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jComboBox_TMP006_ConvRateActionPerformed(evt);
            }
        });

        jLabel6.setFont(new java.awt.Font("Arial", 0, 10)); // NOI18N
        jLabel6.setText("<html> <center>Mode of Operation</center></html>");

        jComboBox_TMP006_MOD.setFont(new java.awt.Font("Arial", 0, 10)); // NOI18N
        jComboBox_TMP006_MOD.setModel(new javax.swing.DefaultComboBoxModel(new String[] { "Power-Down", "Continuous Conversion" }));
        jComboBox_TMP006_MOD.setEnabled(false);
        jComboBox_TMP006_MOD.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jComboBox_TMP006_MODActionPerformed(evt);
            }
        });

        jCheckBox_TMP006_DRDY.setFont(new java.awt.Font("Arial", 0, 10)); // NOI18N
        jCheckBox_TMP006_DRDY.setText("Enable DRDY Interrupts");
        jCheckBox_TMP006_DRDY.setEnabled(false);
        jCheckBox_TMP006_DRDY.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jCheckBox_TMP006_DRDYActionPerformed(evt);
            }
        });

        Button_TMP006_ReadV.setFont(new java.awt.Font("Arial", 0, 10)); // NOI18N
        Button_TMP006_ReadV.setText("<html> <center>Read TMP006  Sensor Voltage</center></html>");
        Button_TMP006_ReadV.setEnabled(false);
        Button_TMP006_ReadV.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                Button_TMP006_ReadVActionPerformed(evt);
            }
        });

        Text_TMP006_VHex.setEditable(false);
        Text_TMP006_VHex.setFont(new java.awt.Font("Arial", 0, 10)); // NOI18N
        Text_TMP006_VHex.setEnabled(false);

        Text_TMP006_THex.setEditable(false);
        Text_TMP006_THex.setFont(new java.awt.Font("Arial", 0, 10)); // NOI18N
        Text_TMP006_THex.setEnabled(false);

        Button_TMP006_ReadT.setFont(new java.awt.Font("Arial", 0, 10)); // NOI18N
        Button_TMP006_ReadT.setText("<html> <center>Read TMP006  Ambient Temp</center></html>");
        Button_TMP006_ReadT.setEnabled(false);
        Button_TMP006_ReadT.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                Button_TMP006_ReadTActionPerformed(evt);
            }
        });

        Text_TMP006_TC.setEditable(false);
        Text_TMP006_TC.setFont(new java.awt.Font("Arial", 0, 10)); // NOI18N
        Text_TMP006_TC.setEnabled(false);

        Text_TMP006_Volt.setEditable(false);
        Text_TMP006_Volt.setFont(new java.awt.Font("Arial", 0, 10)); // NOI18N
        Text_TMP006_Volt.setEnabled(false);

        Icon_TMP006.setFont(new java.awt.Font("Arial", 0, 11)); // NOI18N
        Icon_TMP006.setIcon(new javax.swing.ImageIcon(getClass().getResource("/Resources/red.png"))); // NOI18N
        Icon_TMP006.setText("TMP006");

        javax.swing.GroupLayout jPanelTMP006Layout = new javax.swing.GroupLayout(jPanelTMP006);
        jPanelTMP006.setLayout(jPanelTMP006Layout);
        jPanelTMP006Layout.setHorizontalGroup(
            jPanelTMP006Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanelTMP006Layout.createSequentialGroup()
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                .addGroup(jPanelTMP006Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(jCheckBox_TMP006_DRDY)
                    .addGroup(jPanelTMP006Layout.createSequentialGroup()
                        .addGroup(jPanelTMP006Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                            .addComponent(Button_TMP006_ReadV, javax.swing.GroupLayout.PREFERRED_SIZE, 120, javax.swing.GroupLayout.PREFERRED_SIZE)
                            .addComponent(Button_TMP006_ReadT, javax.swing.GroupLayout.Alignment.TRAILING, javax.swing.GroupLayout.PREFERRED_SIZE, 120, javax.swing.GroupLayout.PREFERRED_SIZE))
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addGroup(jPanelTMP006Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                            .addComponent(Text_TMP006_VHex, javax.swing.GroupLayout.PREFERRED_SIZE, 80, javax.swing.GroupLayout.PREFERRED_SIZE)
                            .addComponent(Text_TMP006_TC, javax.swing.GroupLayout.PREFERRED_SIZE, 80, javax.swing.GroupLayout.PREFERRED_SIZE)
                            .addComponent(Text_TMP006_THex, javax.swing.GroupLayout.PREFERRED_SIZE, 80, javax.swing.GroupLayout.PREFERRED_SIZE)
                            .addComponent(Text_TMP006_Volt, javax.swing.GroupLayout.PREFERRED_SIZE, 80, javax.swing.GroupLayout.PREFERRED_SIZE)))
                    .addGroup(jPanelTMP006Layout.createSequentialGroup()
                        .addGroup(jPanelTMP006Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                            .addComponent(jLabel2, javax.swing.GroupLayout.PREFERRED_SIZE, 90, javax.swing.GroupLayout.PREFERRED_SIZE)
                            .addComponent(jLabel3)
                            .addComponent(jLabel4)
                            .addComponent(jLabel6, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                        .addGap(18, 18, 18)
                        .addGroup(jPanelTMP006Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING, false)
                            .addComponent(Text_TMP006_DEVICEID, javax.swing.GroupLayout.PREFERRED_SIZE, 80, javax.swing.GroupLayout.PREFERRED_SIZE)
                            .addComponent(Text_TMP006_MANUFID, javax.swing.GroupLayout.PREFERRED_SIZE, 80, javax.swing.GroupLayout.PREFERRED_SIZE)
                            .addComponent(jComboBox_TMP006_ConvRate, 0, 110, Short.MAX_VALUE)
                            .addComponent(jComboBox_TMP006_MOD, 0, 1, Short.MAX_VALUE))))
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
            .addGroup(jPanelTMP006Layout.createSequentialGroup()
                .addContainerGap()
                .addComponent(Icon_TMP006)
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );
        jPanelTMP006Layout.setVerticalGroup(
            jPanelTMP006Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanelTMP006Layout.createSequentialGroup()
                .addContainerGap()
                .addComponent(Icon_TMP006)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                .addGroup(jPanelTMP006Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(jLabel3)
                    .addComponent(Text_TMP006_MANUFID, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(jPanelTMP006Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(jLabel4)
                    .addComponent(Text_TMP006_DEVICEID, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(jPanelTMP006Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(jComboBox_TMP006_ConvRate, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(jLabel2, javax.swing.GroupLayout.PREFERRED_SIZE, 20, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addGap(7, 7, 7)
                .addGroup(jPanelTMP006Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(jLabel6, javax.swing.GroupLayout.PREFERRED_SIZE, 20, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(jComboBox_TMP006_MOD, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(jCheckBox_TMP006_DRDY)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                .addGroup(jPanelTMP006Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING, false)
                    .addComponent(Button_TMP006_ReadV, javax.swing.GroupLayout.PREFERRED_SIZE, 57, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addGroup(jPanelTMP006Layout.createSequentialGroup()
                        .addComponent(Text_TMP006_VHex, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                        .addComponent(Text_TMP006_Volt, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)))
                .addGroup(jPanelTMP006Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING, false)
                    .addGroup(jPanelTMP006Layout.createSequentialGroup()
                        .addGap(12, 12, 12)
                        .addComponent(Text_TMP006_THex, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                        .addComponent(Text_TMP006_TC, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addContainerGap())
                    .addGroup(jPanelTMP006Layout.createSequentialGroup()
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(Button_TMP006_ReadT, javax.swing.GroupLayout.PREFERRED_SIZE, 57, javax.swing.GroupLayout.PREFERRED_SIZE))))
        );

        jPanelComm.setBorder(javax.swing.BorderFactory.createBevelBorder(javax.swing.border.BevelBorder.RAISED));

        jLabel5.setFont(new java.awt.Font("Arial", 0, 10)); // NOI18N
        jLabel5.setText("Select Serial Port");

        cboxPorts.setFont(new java.awt.Font("Arial", 0, 10)); // NOI18N

        btnConnect.setFont(new java.awt.Font("Arial", 0, 10)); // NOI18N
        btnConnect.setText("Connect");
        btnConnect.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                btnConnectActionPerformed(evt);
            }
        });

        btnDisconnect.setFont(new java.awt.Font("Arial", 0, 10)); // NOI18N
        btnDisconnect.setText("Disconnect");
        btnDisconnect.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                btnDisconnectActionPerformed(evt);
            }
        });

        Icon_Comm.setIcon(new javax.swing.ImageIcon(getClass().getResource("/Resources/disc.gif"))); // NOI18N

        javax.swing.GroupLayout jPanelCommLayout = new javax.swing.GroupLayout(jPanelComm);
        jPanelComm.setLayout(jPanelCommLayout);
        jPanelCommLayout.setHorizontalGroup(
            jPanelCommLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanelCommLayout.createSequentialGroup()
                .addContainerGap()
                .addGroup(jPanelCommLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addGroup(jPanelCommLayout.createSequentialGroup()
                        .addComponent(jLabel5)
                        .addGap(18, 18, 18)
                        .addComponent(cboxPorts, javax.swing.GroupLayout.PREFERRED_SIZE, 101, javax.swing.GroupLayout.PREFERRED_SIZE))
                    .addGroup(jPanelCommLayout.createSequentialGroup()
                        .addGroup(jPanelCommLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                            .addComponent(btnConnect, javax.swing.GroupLayout.PREFERRED_SIZE, 85, javax.swing.GroupLayout.PREFERRED_SIZE)
                            .addComponent(btnDisconnect))
                        .addGap(18, 18, 18)
                        .addComponent(Icon_Comm)))
                .addContainerGap(26, Short.MAX_VALUE))
        );
        jPanelCommLayout.setVerticalGroup(
            jPanelCommLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, jPanelCommLayout.createSequentialGroup()
                .addContainerGap()
                .addGroup(jPanelCommLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(jLabel5, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                    .addComponent(cboxPorts, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(jPanelCommLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addGroup(jPanelCommLayout.createSequentialGroup()
                        .addComponent(btnConnect)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(btnDisconnect))
                    .addComponent(Icon_Comm))
                .addContainerGap())
        );

        jPanelSMB.setBorder(javax.swing.BorderFactory.createBevelBorder(javax.swing.border.BevelBorder.RAISED));

        Icon_Status.setFont(new java.awt.Font("Arial", 0, 11)); // NOI18N
        Icon_Status.setIcon(new javax.swing.ImageIcon(getClass().getResource("/Resources/red.png"))); // NOI18N
        Icon_Status.setText("Command Status");
        Icon_Status.setEnabled(false);

        jLabel7.setText("SMBus Communication");

        jTextArea_SMBLog.setColumns(20);
        jTextArea_SMBLog.setLineWrap(true);
        jTextArea_SMBLog.setRows(5);
        jTextArea_SMBLog.setWrapStyleWord(true);
        jTextArea_SMBLog.setEnabled(false);
        jScrollPane1.setViewportView(jTextArea_SMBLog);

        jCheckBox_SMBLog.setSelected(true);
        jCheckBox_SMBLog.setText("Enable Log");
        jCheckBox_SMBLog.setEnabled(false);

        jButton_ClearSMBLog.setText("Clear Log");
        jButton_ClearSMBLog.setEnabled(false);
        jButton_ClearSMBLog.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButton_ClearSMBLogActionPerformed(evt);
            }
        });

        javax.swing.GroupLayout jPanelSMBLayout = new javax.swing.GroupLayout(jPanelSMB);
        jPanelSMB.setLayout(jPanelSMBLayout);
        jPanelSMBLayout.setHorizontalGroup(
            jPanelSMBLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanelSMBLayout.createSequentialGroup()
                .addGroup(jPanelSMBLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addGroup(jPanelSMBLayout.createSequentialGroup()
                        .addGap(58, 58, 58)
                        .addComponent(jLabel7)
                        .addGap(0, 0, Short.MAX_VALUE))
                    .addGroup(jPanelSMBLayout.createSequentialGroup()
                        .addContainerGap()
                        .addComponent(jCheckBox_SMBLog)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED, 69, Short.MAX_VALUE)
                        .addComponent(jButton_ClearSMBLog))
                    .addGroup(jPanelSMBLayout.createSequentialGroup()
                        .addContainerGap()
                        .addGroup(jPanelSMBLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                            .addComponent(jScrollPane1, javax.swing.GroupLayout.Alignment.TRAILING)
                            .addComponent(Icon_Status, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))))
                .addContainerGap())
        );
        jPanelSMBLayout.setVerticalGroup(
            jPanelSMBLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanelSMBLayout.createSequentialGroup()
                .addComponent(jLabel7)
                .addGap(8, 8, 8)
                .addComponent(Icon_Status)
                .addGap(1, 1, 1)
                .addComponent(jScrollPane1, javax.swing.GroupLayout.PREFERRED_SIZE, 168, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(jPanelSMBLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(jButton_ClearSMBLog)
                    .addComponent(jCheckBox_SMBLog))
                .addContainerGap(12, Short.MAX_VALUE))
        );

        jPanel_AmbienTemp.setBorder(new javax.swing.border.SoftBevelBorder(javax.swing.border.BevelBorder.RAISED));
        jPanel_AmbienTemp.setMaximumSize(new java.awt.Dimension(300, 300));
        jPanel_AmbienTemp.setPreferredSize(new java.awt.Dimension(200, 200));

        javax.swing.GroupLayout jPanel_AmbienTempLayout = new javax.swing.GroupLayout(jPanel_AmbienTemp);
        jPanel_AmbienTemp.setLayout(jPanel_AmbienTempLayout);
        jPanel_AmbienTempLayout.setHorizontalGroup(
            jPanel_AmbienTempLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGap(0, 0, Short.MAX_VALUE)
        );
        jPanel_AmbienTempLayout.setVerticalGroup(
            jPanel_AmbienTempLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGap(0, 0, Short.MAX_VALUE)
        );

        jPanel_ObjectTemp.setBorder(new javax.swing.border.SoftBevelBorder(javax.swing.border.BevelBorder.RAISED));
        jPanel_ObjectTemp.setMaximumSize(new java.awt.Dimension(300, 300));
        jPanel_ObjectTemp.setPreferredSize(new java.awt.Dimension(200, 200));

        javax.swing.GroupLayout jPanel_ObjectTempLayout = new javax.swing.GroupLayout(jPanel_ObjectTemp);
        jPanel_ObjectTemp.setLayout(jPanel_ObjectTempLayout);
        jPanel_ObjectTempLayout.setHorizontalGroup(
            jPanel_ObjectTempLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGap(0, 0, Short.MAX_VALUE)
        );
        jPanel_ObjectTempLayout.setVerticalGroup(
            jPanel_ObjectTempLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGap(0, 0, Short.MAX_VALUE)
        );

        jLabel1.setFont(new java.awt.Font("Arial", 1, 24)); // NOI18N
        jLabel1.setText("SMBus TI Design");

        javax.swing.GroupLayout jPanel3Layout = new javax.swing.GroupLayout(jPanel3);
        jPanel3.setLayout(jPanel3Layout);
        jPanel3Layout.setHorizontalGroup(
            jPanel3Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel3Layout.createSequentialGroup()
                .addGap(265, 265, 265)
                .addComponent(jLabel1, javax.swing.GroupLayout.PREFERRED_SIZE, 229, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );
        jPanel3Layout.setVerticalGroup(
            jPanel3Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel3Layout.createSequentialGroup()
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                .addComponent(jLabel1))
        );

        jPanel4.setBorder(javax.swing.BorderFactory.createBevelBorder(javax.swing.border.BevelBorder.RAISED));

        jLabel8.setFont(new java.awt.Font("Arial", 0, 10)); // NOI18N
        jLabel8.setText("Device ID");

        Text_DIM430_DEVICEID.setEditable(false);
        Text_DIM430_DEVICEID.setFont(new java.awt.Font("Arial", 0, 10)); // NOI18N
        Text_DIM430_DEVICEID.setEnabled(false);

        jCheckBox_DIM430_OE.setFont(new java.awt.Font("Arial", 0, 10)); // NOI18N
        jCheckBox_DIM430_OE.setText("Enable Dimmer Output");
        jCheckBox_DIM430_OE.setEnabled(false);
        jCheckBox_DIM430_OE.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jCheckBox_DIM430_OEActionPerformed(evt);
            }
        });

        jSlider_DIM430_Duty.setValue(0);
        jSlider_DIM430_Duty.setEnabled(false);
        jSlider_DIM430_Duty.addChangeListener(new javax.swing.event.ChangeListener() {
            public void stateChanged(javax.swing.event.ChangeEvent evt) {
                jSlider_DIM430_DutyStateChanged(evt);
            }
        });

        jLabel_DIM430Duty.setText("PWM Duty Cycle");

        Icon_DIM430.setFont(new java.awt.Font("Arial", 0, 11)); // NOI18N
        Icon_DIM430.setIcon(new javax.swing.ImageIcon(getClass().getResource("/Resources/red.png"))); // NOI18N
        Icon_DIM430.setText("DIM430");

        javax.swing.GroupLayout jPanel4Layout = new javax.swing.GroupLayout(jPanel4);
        jPanel4.setLayout(jPanel4Layout);
        jPanel4Layout.setHorizontalGroup(
            jPanel4Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel4Layout.createSequentialGroup()
                .addContainerGap()
                .addGroup(jPanel4Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addGroup(jPanel4Layout.createSequentialGroup()
                        .addComponent(jLabel8)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(Text_DIM430_DEVICEID, javax.swing.GroupLayout.PREFERRED_SIZE, 80, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addGap(0, 0, Short.MAX_VALUE))
                    .addComponent(jCheckBox_DIM430_OE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                    .addGroup(jPanel4Layout.createSequentialGroup()
                        .addGroup(jPanel4Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                            .addComponent(jLabel_DIM430Duty)
                            .addComponent(Icon_DIM430)
                            .addComponent(jSlider_DIM430_Duty, javax.swing.GroupLayout.PREFERRED_SIZE, 222, javax.swing.GroupLayout.PREFERRED_SIZE))
                        .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))))
        );
        jPanel4Layout.setVerticalGroup(
            jPanel4Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel4Layout.createSequentialGroup()
                .addGap(4, 4, 4)
                .addComponent(Icon_DIM430)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED, 27, Short.MAX_VALUE)
                .addGroup(jPanel4Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(jLabel8)
                    .addComponent(Text_DIM430_DEVICEID, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addGap(18, 18, 18)
                .addComponent(jCheckBox_DIM430_OE)
                .addGap(18, 18, 18)
                .addComponent(jLabel_DIM430Duty)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                .addComponent(jSlider_DIM430_Duty, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addContainerGap())
        );

        jPanel_BlockDiagram.setBorder(javax.swing.BorderFactory.createBevelBorder(javax.swing.border.BevelBorder.RAISED));
        jPanel_BlockDiagram.setAutoscrolls(true);
        jPanel_BlockDiagram.setLayout(new org.netbeans.lib.awtextra.AbsoluteLayout());

        Image_BD_PC.setIcon(new javax.swing.ImageIcon(getClass().getResource("/Resources/PC_Off.png"))); // NOI18N
        jPanel_BlockDiagram.add(Image_BD_PC, new org.netbeans.lib.awtextra.AbsoluteConstraints(10, 10, -1, -1));

        Image_BD_PCtoMaster.setIcon(new javax.swing.ImageIcon(getClass().getResource("/Resources/PC_to_Master_Off.png"))); // NOI18N
        jPanel_BlockDiagram.add(Image_BD_PCtoMaster, new org.netbeans.lib.awtextra.AbsoluteConstraints(20, 30, -1, -1));

        Image_BD_MastertoPC.setIcon(new javax.swing.ImageIcon(getClass().getResource("/Resources/Master_to_PC_Off.png"))); // NOI18N
        jPanel_BlockDiagram.add(Image_BD_MastertoPC, new org.netbeans.lib.awtextra.AbsoluteConstraints(50, 30, -1, -1));

        Image_BD_Master.setIcon(new javax.swing.ImageIcon(getClass().getResource("/Resources/Master_Off.png"))); // NOI18N
        jPanel_BlockDiagram.add(Image_BD_Master, new org.netbeans.lib.awtextra.AbsoluteConstraints(10, 60, -1, -1));

        Image_BD_SMBus.setIcon(new javax.swing.ImageIcon(getClass().getResource("/Resources/SMBus_Off.png"))); // NOI18N
        jPanel_BlockDiagram.add(Image_BD_SMBus, new org.netbeans.lib.awtextra.AbsoluteConstraints(80, 60, -1, -1));

        Image_BD_HosttoSlave.setIcon(new javax.swing.ImageIcon(getClass().getResource("/Resources/Down_Off.png"))); // NOI18N
        jPanel_BlockDiagram.add(Image_BD_HosttoSlave, new org.netbeans.lib.awtextra.AbsoluteConstraints(100, 70, -1, -1));

        Image_BD_Slave.setIcon(new javax.swing.ImageIcon(getClass().getResource("/Resources/Slave_Off.png"))); // NOI18N
        jPanel_BlockDiagram.add(Image_BD_Slave, new org.netbeans.lib.awtextra.AbsoluteConstraints(80, 100, -1, 30));

        Image_BD_SlavetoHost.setIcon(new javax.swing.ImageIcon(getClass().getResource("/Resources/Up_Off.png"))); // NOI18N
        jPanel_BlockDiagram.add(Image_BD_SlavetoHost, new org.netbeans.lib.awtextra.AbsoluteConstraints(120, 80, -1, -1));

        Image_BD_TMP006.setIcon(new javax.swing.ImageIcon(getClass().getResource("/Resources/TMP006_Off.png"))); // NOI18N
        jPanel_BlockDiagram.add(Image_BD_TMP006, new org.netbeans.lib.awtextra.AbsoluteConstraints(150, 100, -1, 30));

        Image_BD_TMP006toHost.setIcon(new javax.swing.ImageIcon(getClass().getResource("/Resources/Up_Off.png"))); // NOI18N
        jPanel_BlockDiagram.add(Image_BD_TMP006toHost, new org.netbeans.lib.awtextra.AbsoluteConstraints(190, 80, -1, -1));

        Image_BD_HosttoTMP006.setIcon(new javax.swing.ImageIcon(getClass().getResource("/Resources/Down_Off.png"))); // NOI18N
        jPanel_BlockDiagram.add(Image_BD_HosttoTMP006, new org.netbeans.lib.awtextra.AbsoluteConstraints(170, 70, -1, -1));

        javax.swing.GroupLayout layout = new javax.swing.GroupLayout(this);
        this.setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addContainerGap()
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addGroup(layout.createSequentialGroup()
                        .addComponent(jLabel12, javax.swing.GroupLayout.PREFERRED_SIZE, 735, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addGap(0, 26, Short.MAX_VALUE))
                    .addGroup(layout.createSequentialGroup()
                        .addComponent(jPanel3, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                        .addContainerGap())
                    .addGroup(layout.createSequentialGroup()
                        .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING, false)
                            .addComponent(jPanelTMP006, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                            .addComponent(jPanel4, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                        .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.CENTER)
                            .addComponent(jPanel_AmbienTemp, javax.swing.GroupLayout.DEFAULT_SIZE, 260, Short.MAX_VALUE)
                            .addComponent(jPanel_ObjectTemp, javax.swing.GroupLayout.DEFAULT_SIZE, 260, Short.MAX_VALUE))
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                        .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING, false)
                            .addComponent(jPanelComm, javax.swing.GroupLayout.Alignment.TRAILING, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                            .addComponent(jPanelSMB, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                            .addComponent(jPanel_BlockDiagram, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)))))
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, layout.createSequentialGroup()
                .addComponent(jPanel3, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING)
                    .addGroup(layout.createSequentialGroup()
                        .addGap(0, 0, Short.MAX_VALUE)
                        .addComponent(jPanelTMP006, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addGap(18, 18, Short.MAX_VALUE)
                        .addComponent(jPanel4, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                    .addGroup(javax.swing.GroupLayout.Alignment.LEADING, layout.createSequentialGroup()
                        .addComponent(jPanelComm, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED, 18, Short.MAX_VALUE)
                        .addComponent(jPanelSMB, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                        .addComponent(jPanel_BlockDiagram, javax.swing.GroupLayout.PREFERRED_SIZE, 142, javax.swing.GroupLayout.PREFERRED_SIZE))
                    .addGroup(javax.swing.GroupLayout.Alignment.LEADING, layout.createSequentialGroup()
                        .addComponent(jPanel_AmbienTemp, javax.swing.GroupLayout.DEFAULT_SIZE, 262, Short.MAX_VALUE)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(jPanel_ObjectTemp, javax.swing.GroupLayout.DEFAULT_SIZE, 260, Short.MAX_VALUE)))
                .addGap(11, 11, 11)
                .addComponent(jLabel12))
        );
    }// </editor-fold>//GEN-END:initComponents

    private void btnConnectActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_btnConnectActionPerformed
        MyCommSM.communicator.connect();
        if (MyCommSM.communicator.getConnected() == true)
        {
            if (MyCommSM.communicator.initIOStream() == true)
            {
                MyCommSM.communicator.initListener();
                Icon_Comm.setIcon(new ImageIcon(getClass().getResource("/Resources/conn.gif")));
                Icon_Status.setEnabled(true);
                //jCheckBoxTMP006.setEnabled(true);
                TMP006_comm.Enable();
                DIM430_comm.Enable();
                jTextArea_SMBLog.setEnabled(true);
                jCheckBox_SMBLog.setEnabled(true);
                jButton_ClearSMBLog.setEnabled(true);
            }
        }
    }//GEN-LAST:event_btnConnectActionPerformed

    private void btnDisconnectActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_btnDisconnectActionPerformed
        MyCommSM.communicator.disconnect();
        Icon_Comm.setIcon(new ImageIcon(getClass().getResource("/Resources/disc.gif")));
        Icon_Status.setEnabled(false);
        TMP006_comm.Disable();
        DIM430_comm.Disable();
        jTextArea_SMBLog.setEnabled(false);
        jCheckBox_SMBLog.setEnabled(false);
        jButton_ClearSMBLog.setEnabled(false);
        MyCommSM.TX_Cmd = 0;
        MyCommSM.RX_Counter = 0;
        MyCommSM.WaitCmd = false;
        MyCommSM.SM = GUI.Response_SM.SM_WAIT_HDR;
        MyCommSM.RX_Len = 0;
        MyCommSM.RX_Cmd = 0;
        
    }//GEN-LAST:event_btnDisconnectActionPerformed

    private static final char RESPONSE_OK = 0x00;
    private static final char RESPONSE_RX_ERROR = 0x01;
    private static final char RESPONSE_SMB_ERROR = 0x02;

    public enum Response_SM {
    SM_WAIT_HDR,
    SM_WAIT_LEN,
    SM_WAIT_CTRL,
    SM_WAIT_STATUS,
    SM_WAIT_TXLEN,
    SM_WAIT_TXDATA,
    SM_WAIT_RESPCMD,
    SM_WAIT_RESPDATA,
    SM_DONE,
    SM_ERROR,
    SM_BUSY,
    };

     
    
   
    private int GetPayload(byte[] Payload)
    {
        return MyCommSM.communicator.ReadData(Payload);
    }
    
    
    
    
    public static void ProcessResponse(byte response) throws InterruptedException
    {
        Block_Master.Blink(200);
        Block_MastertoPC.Blink(200);
        try
        {
             MyCommSM.mutex.acquire();
             switch (MyCommSM.SM)
            {
                case SM_WAIT_HDR:
                case SM_DONE:
                case SM_ERROR:
                    // 1st byte is header
                    if (response == (byte) 0x80)
                    {
                        MyCommSM.RX_Counter = 0; 
                        MyCommSM.RX_Len= 0; 
                        MyCommSM.SM = Response_SM.SM_WAIT_LEN;
                    }
                    else
                        MyCommSM.SM = Response_SM.SM_ERROR;
                    break;
                case SM_WAIT_LEN:
                    // 2nd byte is Len
                    if ((response >= 3) && (response < 37))
                    {
                        MyCommSM.RX_Len = response;
                        MyCommSM.SM = Response_SM.SM_WAIT_CTRL;
                    }
                    else
                        MyCommSM.SM = Response_SM.SM_ERROR;
                    break;
                case SM_WAIT_CTRL:
                    //3rd byte is Ctrl
                    // For now, just ignore Ctrl
                    MyCommSM.SM = Response_SM.SM_WAIT_STATUS;
                    break;
                case SM_WAIT_STATUS:
                    // 4th byte is Ctrl
                    // For now, just ignore Status
                    MyCommSM.SM = Response_SM.SM_WAIT_TXLEN;
                    break;
                case SM_WAIT_TXLEN:
                    // 6th byte is TxLen
                    MyCommSM.SMB_TX_Len = response;
                    if (response > 0)
                        MyCommSM.SM = Response_SM.SM_WAIT_TXDATA;
                    else
                        MyCommSM.SM = Response_SM.SM_WAIT_RESPCMD;
                    break;
                case SM_WAIT_TXDATA:
                    // 7th byte starts SMB TX data
                    if (MyCommSM.RX_Counter < MyCommSM.SMB_TX_Buff.length)
                    {
                        MyCommSM.SMB_TX_Buff[MyCommSM.RX_Counter] = response;
                    }
                    MyCommSM.RX_Counter++;
                    if (MyCommSM.RX_Counter >= MyCommSM.SMB_TX_Len)
                    {
                        MyCommSM.SM = Response_SM.SM_WAIT_RESPCMD;
                        MyCommSM.RX_Counter = 0; //restart counter to get response
                    }
                break;
                case SM_WAIT_RESPCMD:
                    // 5th byte is Command
                    MyCommSM.RX_Cmd = response;
                    MyCommSM.SM = Response_SM.SM_WAIT_RESPDATA;
                    break;
                case SM_WAIT_RESPDATA:
                    // 6th-n bytes is Data
                    if (MyCommSM.RX_Counter < MyCommSM.SMB_Resp_Buff.length)
                        MyCommSM.SMB_Resp_Buff[MyCommSM.RX_Counter++] = response;
                    break;
            }
            // Check if the packet is complete

            if ((MyCommSM.SM == Response_SM.SM_WAIT_RESPDATA) && (MyCommSM.RX_Counter == MyCommSM.RX_Len-(4+MyCommSM.SMB_TX_Len)))
            {
                Block_SMbus.Blink(100);
                Block_Master.Blink(100);
                // Display SMBus TX data
                if (MyCommSM.SMB_TX_Len >= 2)
                {
                    if (MyCommSM.SMB_TX_Buff[0] == 0x43)
                    {
                        // Data from DIM430
                        AppendtoLog("Master to DIM430", MyCommSM.SMB_TX_Buff, MyCommSM.SMB_TX_Len);
                        Block_Slave.Blink(100);
                        Block_SMBtoSlave.Blink(100);
                        // Print response if any
                        if (MyCommSM.RX_Counter >0)
                        {
                            Block_SlavetoSMB.Blink(100);
                            AppendtoLog("DIM430 to Master", MyCommSM.SMB_Resp_Buff, MyCommSM.RX_Counter);
                        }
                    }
                    else if (MyCommSM.SMB_TX_Buff[0] == 0x40)
                    {
                        // Data from TMP006
                        AppendtoLog("Master to TMP006", MyCommSM.SMB_TX_Buff, MyCommSM.SMB_TX_Len);
                        Block_TMP006.Blink(100);
                        Block_SMBtoTMP006.Blink(100);
                        // Print response if any
                        if (MyCommSM.RX_Counter >0)
                        {
                            Block_TMP006toSMB.Blink(100);
                            AppendtoLog("TMP006 to Master", MyCommSM.SMB_Resp_Buff, MyCommSM.RX_Counter);
                        }
                    }
                }
                
                // Process the packet
                switch (MyCommSM.RX_Cmd)
                {
                    case RESPONSE_OK:
                        Icon_Status.setIcon(new ImageIcon(ImageIcon.class.getResource("/Resources/green.png")));
                         MyCommSM.SM = Response_SM.SM_DONE;  // Packet reception is complete
                        break;
                    case TMP006_Comm.RESPONSE_TMP006_MANID:
                    case TMP006_Comm.RESPONSE_TMP006_CONFIG:
                    case TMP006_Comm.RESPONSE_TMP006_DEVID:
                    case TMP006_Comm.RESPONSE_TMP006_T:
                    case TMP006_Comm.RESPONSE_TMP006_V:
                        TMP006_comm.ProcessResponse();
                        break;
                    case DIM430_Comm.RESPONSE_DIM430_DEVID:
                    case DIM430_Comm.RESPONSE_DIM430_CONFIG:
                    case DIM430_Comm.RESPONSE_DIM430_DUTY:
                        DIM430_comm.ProcessResponse();
                        break;
                    default:
                        MyCommSM.SM = Response_SM.SM_ERROR;
                        break;

                }
            }
        } finally{
            MyCommSM.mutex.release();
        }
    }
    
    private static void AppendtoLog(String flow, byte[] Data, int len)
    {
        byte array[] = new byte[len];
        System.arraycopy(Data, 0, array, 0, len);
        
        if (jCheckBox_SMBLog.isSelected() == true)
        {
            try {
                DateFormat dateFormat = new SimpleDateFormat("HH:mm:ss");
                Date date = new Date();
                GUI.jTextArea_SMBLog.getDocument().insertString(0, flow + " ("+  
                                                        dateFormat.format(date) +")\n"+
                                                        Slave_Comm.BytesToASCII(array) +"\n", null);
            } catch (BadLocationException e1) {
                // TODO Auto-generated catch block
                e1.printStackTrace();
            }
        }
        
    }
    
    private void Button_TMP006_ReadTActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_Button_TMP006_ReadTActionPerformed
        TMP006_comm.Read_AmbientTemp();
    }//GEN-LAST:event_Button_TMP006_ReadTActionPerformed


    private void Button_TMP006_ReadVActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_Button_TMP006_ReadVActionPerformed
        TMP006_comm.Read_Voltage();
    }//GEN-LAST:event_Button_TMP006_ReadVActionPerformed

          
    private void jCheckBox_TMP006_DRDYActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jCheckBox_TMP006_DRDYActionPerformed
        if (jCheckBox_TMP006_DRDY.isEnabled() == true)
             TMP006_comm.Change_Config();
    }//GEN-LAST:event_jCheckBox_TMP006_DRDYActionPerformed

    private void jComboBox_TMP006_ConvRateActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jComboBox_TMP006_ConvRateActionPerformed
        if (jComboBox_TMP006_ConvRate.isEnabled() == true)
             TMP006_comm.Change_Config();
    }//GEN-LAST:event_jComboBox_TMP006_ConvRateActionPerformed

    private void jComboBox_TMP006_MODActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jComboBox_TMP006_MODActionPerformed
         if (jComboBox_TMP006_MOD.isEnabled() == true)
              TMP006_comm.Change_Config();
    }//GEN-LAST:event_jComboBox_TMP006_MODActionPerformed

    private void jCheckBox_DIM430_OEActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jCheckBox_DIM430_OEActionPerformed
         DIM430_comm.Change_Config();
    }//GEN-LAST:event_jCheckBox_DIM430_OEActionPerformed

    private void jSlider_DIM430_DutyStateChanged(javax.swing.event.ChangeEvent evt) {//GEN-FIRST:event_jSlider_DIM430_DutyStateChanged
         DIM430_comm.Change_Duty();
    }//GEN-LAST:event_jSlider_DIM430_DutyStateChanged

    private void jButton_ClearSMBLogActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButton_ClearSMBLogActionPerformed
        jTextArea_SMBLog.setText(null);
    }//GEN-LAST:event_jButton_ClearSMBLogActionPerformed

    public static void Disable_TMP006_GUIComponents()
    {
        Text_TMP006_MANUFID.setText("");        
        Text_TMP006_DEVICEID.setText("");   
        Button_TMP006_ReadT.setEnabled(false);
        Button_TMP006_ReadV.setEnabled(false);
        Text_TMP006_DEVICEID.setEnabled(false);
        Text_TMP006_MANUFID.setEnabled(false);
        jComboBox_TMP006_ConvRate.setEnabled(false);
        jComboBox_TMP006_MOD.setEnabled(false);
        jCheckBox_TMP006_DRDY.setEnabled(false);
        Text_TMP006_THex.setEnabled(false);
        Text_TMP006_VHex.setEnabled(false);
        ChartPanel Chart = (ChartPanel)jPanel_AmbienTemp;
        MeterPlot Plot = (MeterPlot) Chart.getChart().getPlot();
        Plot.setDialBackgroundPaint(Color.lightGray);   
        Plot.setNeedlePaint(Color.lightGray);   
        Plot.setTickLabelPaint(Color.lightGray);   
        Plot.setTickPaint(Color.lightGray);   
        Plot.setValuePaint(Color.lightGray);   
        ChartPanel Chart2 = (ChartPanel)jPanel_ObjectTemp;
        Plot = (MeterPlot) Chart2.getChart().getPlot();
        Plot.setDialBackgroundPaint(Color.lightGray);   
        Plot.setNeedlePaint(Color.lightGray);   
        Plot.setTickLabelPaint(Color.lightGray);   
        Plot.setTickPaint(Color.lightGray);   
        Plot.setValuePaint(Color.lightGray);   
    }
    
     public static void Enable_TMP006_GUIComponents()
    {
        Button_TMP006_ReadT.setEnabled(true);
        Button_TMP006_ReadV.setEnabled(true);
        Text_TMP006_DEVICEID.setEnabled(true);
        Text_TMP006_MANUFID.setEnabled(true);
        jComboBox_TMP006_ConvRate.setEnabled(true);
        jComboBox_TMP006_MOD.setEnabled(true);
        jCheckBox_TMP006_DRDY.setEnabled(true);
        Text_TMP006_THex.setEnabled(true);
        Text_TMP006_VHex.setEnabled(true);
        
        ChartPanel Chart = (ChartPanel)jPanel_AmbienTemp;
        MeterPlot Plot = (MeterPlot) Chart.getChart().getPlot();
        Plot.setDialBackgroundPaint(Color.white);   
        Plot.setNeedlePaint(Color.BLACK);   
        Plot.setTickLabelPaint(Color.darkGray);   
        Plot.setTickPaint(Color.BLACK);   
        Plot.setValuePaint(Color.black);   
        ChartPanel Chart2 = (ChartPanel)jPanel_ObjectTemp;
        Plot = (MeterPlot) Chart2.getChart().getPlot();
        Plot.setDialBackgroundPaint(Color.white);   
        Plot.setNeedlePaint(Color.BLACK);   
        Plot.setTickLabelPaint(Color.darkGray);   
        Plot.setTickPaint(Color.BLACK);   
        Plot.setValuePaint(Color.black);   
    }
     
     public static void Disable_DIM430_GUIComponents()
    {
        Text_DIM430_DEVICEID.setText("");
        Text_DIM430_DEVICEID.setEnabled(false);
        jCheckBox_DIM430_OE.setSelected(false);
        jCheckBox_DIM430_OE.setEnabled(false);
        jSlider_DIM430_Duty.setValue(0);
        jSlider_DIM430_Duty.setEnabled(false);
    }

    public static void Enable_DIM430_GUIComponents()
    {
        Text_DIM430_DEVICEID.setEnabled(true);
        jCheckBox_DIM430_OE.setEnabled(true);
        jSlider_DIM430_Duty.setEnabled(true);
    }

    // Variables declaration - do not modify//GEN-BEGIN:variables
    private static javax.swing.JButton Button_TMP006_ReadT;
    private static javax.swing.JButton Button_TMP006_ReadV;
    private javax.swing.JLabel Icon_Comm;
    public static javax.swing.JLabel Icon_DIM430;
    public static javax.swing.JLabel Icon_Status;
    public static javax.swing.JLabel Icon_TMP006;
    private javax.swing.JLabel Image_BD_HosttoSlave;
    private javax.swing.JLabel Image_BD_HosttoTMP006;
    private javax.swing.JLabel Image_BD_Master;
    private javax.swing.JLabel Image_BD_MastertoPC;
    private javax.swing.JLabel Image_BD_PC;
    private javax.swing.JLabel Image_BD_PCtoMaster;
    private javax.swing.JLabel Image_BD_SMBus;
    private javax.swing.JLabel Image_BD_Slave;
    private javax.swing.JLabel Image_BD_SlavetoHost;
    private javax.swing.JLabel Image_BD_TMP006;
    private javax.swing.JLabel Image_BD_TMP006toHost;
    public static javax.swing.JTextField Text_DIM430_DEVICEID;
    public static javax.swing.JTextField Text_TMP006_DEVICEID;
    public static javax.swing.JTextField Text_TMP006_MANUFID;
    public static javax.swing.JTextField Text_TMP006_TC;
    public static javax.swing.JTextField Text_TMP006_THex;
    public static javax.swing.JTextField Text_TMP006_VHex;
    public static javax.swing.JTextField Text_TMP006_Volt;
    public javax.swing.JButton btnConnect;
    public javax.swing.JButton btnDisconnect;
    public javax.swing.JComboBox cboxPorts;
    private javax.swing.JButton jButton_ClearSMBLog;
    public static javax.swing.JCheckBox jCheckBox_DIM430_OE;
    public static javax.swing.JCheckBox jCheckBox_SMBLog;
    public static javax.swing.JCheckBox jCheckBox_TMP006_DRDY;
    public static javax.swing.JComboBox jComboBox_TMP006_ConvRate;
    public static javax.swing.JComboBox jComboBox_TMP006_MOD;
    private javax.swing.JLabel jLabel1;
    private javax.swing.JLabel jLabel12;
    private javax.swing.JLabel jLabel2;
    private javax.swing.JLabel jLabel3;
    private javax.swing.JLabel jLabel4;
    private javax.swing.JLabel jLabel5;
    private javax.swing.JLabel jLabel6;
    private javax.swing.JLabel jLabel7;
    private javax.swing.JLabel jLabel8;
    private javax.swing.JLabel jLabel_DIM430Duty;
    private javax.swing.JPanel jPanel3;
    private javax.swing.JPanel jPanel4;
    private javax.swing.JPanel jPanelComm;
    private javax.swing.JPanel jPanelSMB;
    private javax.swing.JPanel jPanelTMP006;
    private static javax.swing.JPanel jPanel_AmbienTemp;
    private javax.swing.JPanel jPanel_BlockDiagram;
    private static javax.swing.JPanel jPanel_ObjectTemp;
    private javax.swing.JScrollPane jScrollPane1;
    public static javax.swing.JSlider jSlider_DIM430_Duty;
    public static javax.swing.JTextArea jTextArea_SMBLog;
    // End of variables declaration//GEN-END:variables
}



