/* --COPYRIGHT--,BSD
 * Copyright (c) 2014, Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * --/COPYRIGHT--*/
package SMBUsDemo;


import SMBUsDemo.GUI;
import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Font;
import javax.swing.JPanel;
import org.jfree.chart.ChartMouseEvent;
import org.jfree.chart.ChartMouseListener;
import org.jfree.chart.ChartPanel;
import org.jfree.chart.JFreeChart;
import org.jfree.chart.plot.MeterInterval;
import org.jfree.chart.plot.MeterPlot;
import org.jfree.data.Range;
import org.jfree.data.general.DefaultValueDataset;

/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
/**
 *
 * @author a0272600
 */
/*!
* \class MyThermometer
*
* \brief Class used to handle a graph which shows temperature
*
* This class implements a gauge which is used to display temperature in this GUI
*  
*/
public class MyThermometer extends JPanel implements ChartMouseListener{
    public ChartPanel chartPanel;
    public DefaultValueDataset Data;
    private final double minTemp = -10.0; 
    private final double maxTemp = 50; 
    private double left_margin = 10;
    private double right_margin = 30;
    private final int num_ranges = 50;
    public int r =0, g=255, b=0;

    private enum Set_Range_t {
         IDLE,
         SET_LEFT_MARGIN,
         SET_RIGHT_MARGIN,
         };
    Set_Range_t Set_Range;

    public MyThermometer() {
         chartPanel = createThermometer();
         Set_Range = Set_Range_t.IDLE;
     }

    public void SetRanges(double leftm, double rightm)
     {
         double margin = leftm;
         double next_margin; 
         this.left_margin = leftm;
         this.right_margin = rightm;
         MeterPlot Plot = (MeterPlot) this.chartPanel.getChart().getPlot();

         Plot.clearIntervals();
         Plot.addInterval(new MeterInterval("", new Range(this.minTemp, this.left_margin), 
             Color.lightGray, new BasicStroke(2.0F), new Color(0, 0, 0, 60)));
         for (int i =0; i < this.num_ranges; i++)
         {
             next_margin = margin + (this.right_margin-this.left_margin)/this.num_ranges;
             Plot.addInterval(new MeterInterval("", new Range(margin, next_margin), 
                             null, null, new Color(this.r, this.g, this.b, i*150/num_ranges)));
             margin = next_margin;
         }
     Plot.addInterval(new MeterInterval("", new Range(this.right_margin, this.maxTemp), 
         Color.lightGray, new BasicStroke(2.0F), new Color(0, 0, 0, 60)));
     }
    
    public void SetRanges()
     {
         SetRanges(this.left_margin, this.right_margin);
     }
    
    public double GetMinTemp()
    {
        return this.minTemp;
    }
    
    public double GetMaxTemp()
    {
        return this.maxTemp;
    }
    
    public double GetLeftMargin()
    {
        return this.left_margin;
    }
    
    public double GetRightMargin()
    {
        return this.right_margin;
    }

     public void SetTitle(String title)
     {
         this.chartPanel.getChart().setTitle(title);
     }
     
 
     public void SetLeftMargin(double Margin)
     {
         this.left_margin = Margin;
     }

     public void SetRightMargin(double Margin)
     {
         this.right_margin = Margin;
     }
     

     private ChartPanel createThermometer()  {
     Data = new DefaultValueDataset(0.0);
     MeterPlot Plot = new MeterPlot(Data);

     Plot.setRange(new Range(minTemp, maxTemp));
     //SetRanges(Plot);

     Plot.setDialBackgroundPaint(Color.lightGray);   
     Plot.setNeedlePaint(Color.lightGray);   
     Plot.setDialOutlinePaint(Color.gray);
     Plot.setMeterAngle(260);
     Plot.setTickLabelsVisible(true);   
     Plot.setTickLabelFont(new Font("Dialog", 1, 10));   
     Plot.setTickLabelPaint(Color.lightGray);   
     Plot.setTickSize(5D);   
     Plot.setTickPaint(Color.lightGray);   
     Plot.setValuePaint(Color.lightGray);   
     Plot.setValueFont(new Font("Dialog", 1, 14));   
     Plot.setUnits("°C");

     //public class GUI extends JP
     JFreeChart chart = new JFreeChart("",
                                       JFreeChart.DEFAULT_TITLE_FONT,
                                       Plot, 
                                       false);

     chartPanel = new ChartPanel(chart);
    // chartPanel.addChartMouseListener(this);

     return chartPanel;
 }


    @Override
     public void chartMouseClicked(ChartMouseEvent event) {
        
     }

     @Override
     public void chartMouseMoved(ChartMouseEvent event) {
         
     }  
}


