/* --COPYRIGHT--,BSD
 * Copyright (c) 2014, Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * --/COPYRIGHT--*/
/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package SMBUsDemo;

import java.util.Arrays;
import javax.swing.ImageIcon;
import java.util.concurrent.Callable;
import java.util.concurrent.Executors;
import java.util.concurrent.ScheduledExecutorService;
import java.util.concurrent.ScheduledFuture;
import java.util.concurrent.TimeUnit;


/**
 *
 * @author a0272600
 */
/*!
* \class Slave_Comm
*
* \brief Class used to handle generic communication to Slaves (through Master)
*
* This class has generic functions used to send packets to the Master device which
*   will trigger packets to the slaves. The class also handles responses from 
*   slaves to the PC.
*  
*/
public class Slave_Comm  {
    public Boolean is_enabled = false;
    public Boolean is_connected = false;
    java.lang.Object stick;
    java.lang.Object stick_WaitCommand;
    Boolean command_found = false;
    Boolean command_timeout = false;

    
    public Slave_Comm() {
         is_enabled = false;
         is_connected = false;
         stick = new java.lang.Object();
         stick_WaitCommand = new java.lang.Object();
         command_found = false;
         command_timeout = false;
     }
    
     public void Enable() {
         is_enabled = true;
         synchronized (stick) {
            stick.notify();
          }
     }
     
     public synchronized void Disable() {
         is_enabled = false;
         is_connected = false;
     }
     
     final protected static char[] hexConvArray = "0123456789ABCDEF".toCharArray();

     public static String ReverseWordToASCII(byte[] bytes)
    {
        char[] hexArray = new char[4];

        hexArray[0] = hexConvArray[(bytes[1] & 0xFF) >>> 4];
        hexArray[1] = hexConvArray[(bytes[1] & 0xFF) & 0x0F];
        hexArray[2] = hexConvArray[(bytes[0] & 0xFF) >>> 4];
        hexArray[3] = hexConvArray[(bytes[0] & 0xFF) & 0x0F];
        
        return new String(hexArray);
    }
     
    public static String ReverseBytesToASCII(byte[] bytes)
    {
        char[] hexArray = new char[bytes.length *3];
        int count = bytes.length*3 -1;
        for (int i=0; i < bytes.length; i++)
        {
            int temp = (bytes[i] & 0xFF);
            hexArray[count--] = ' ';
            hexArray[count--] = hexConvArray[temp & 0x0F];
            hexArray[count--] = hexConvArray[temp >>> 4];
        }
        return new String(hexArray);
    }
      
    public static String BytesToASCII(byte[] bytes)
    {
        char[] hexArray = new char[bytes.length *3];
        int count = 0;
        for (int i=0; i < bytes.length; i++)
        {
            int temp = (bytes[i] & 0xFF);
            hexArray[count++] = hexConvArray[temp >>> 4];
            hexArray[count++] = hexConvArray[temp & 0x0F];
            hexArray[count++] = ' ';
        }
        return new String(hexArray);
    }
     
      private String ArraytoHexString(byte[] data)
     {
         return this.ReverseBytesToASCII(data);
     }
     private String ArraytoHexString(byte[] data, Boolean reverse)
     {
         if (reverse == true)
             return this.ReverseBytesToASCII(data);
         else
             return this.BytesToASCII(data);
     }
     
     public String Get_ResponseAsString()
    {
        byte[] Resp_Array = this.Get_ResponseAsArray();
        String Resp_String = null;
        
        if (Resp_Array.length == 2)
            Resp_String = "0x"+ReverseWordToASCII(Resp_Array);
        
        return Resp_String;
    }
    
    public byte[] Get_ResponseAsArray()
    {
        byte[] Resp_Array = null;
        GUI.CommInterface Comm = GUI.MyCommSM;
        
        if (is_connected == false)
        {
            Comm.SM = GUI.Response_SM.SM_WAIT_HDR;  // Ignore packet if TMP is disabled
        }
        else if (Comm.RX_Counter == 2)  // 2 bytes are expected
        {
            Resp_Array = new byte[2];
            Resp_Array = Arrays.copyOf(Comm.SMB_Resp_Buff, 2);
        }
        else
        {
            Comm.SM = GUI.Response_SM.SM_ERROR;
        }
        return Resp_Array;
    }
    
    public Boolean ProcessResponse()
    {
        return false;
    }
    
    public boolean SendCommand(byte[] cmd) 
    {
        GUI.CommInterface Comm = GUI.MyCommSM;

        if (cmd.length < 2)
            return false;   // Error in packet

        GUI.Block_PC.Blink(200);
        GUI.Block_PCtoMaster.Blink(200);
        Comm.communicator.writeArray(cmd);  // Packet sent 
        
        return true;
    }
    
    public Boolean SendCommand_WaitUntilDone(byte[] cmd) throws InterruptedException
    {
        GUI.Response_SM Status = GUI.Response_SM.SM_BUSY;
        int RXretries =0;
        Boolean CMD_Found, CMD_Timeout;
        GUI.CommInterface Comm = GUI.MyCommSM;
        
        Boolean done = false;
        do
        {
            try
            {
                Comm.mutex.acquire();
                // If a packet is in reception, wait for it to finish
                if (Comm.WaitCmd == false)
                {
                    SendCommand(cmd);
                    Comm.WaitCmd = true;
                    command_found = false;
                    command_timeout = false;
                    Comm.TX_Cmd = cmd[2];
                    done = true;
                    // Set Status as red in case there's no response, it will be updated upon RX of response
                    GUI.Icon_Status.setIcon(new ImageIcon(getClass().getResource("/Resources/red.png")));
                }
            } finally
            {
                Comm.mutex.release();
            }
        }while (done == false);


        ScheduledExecutorService scheduledExecutorService = Executors.newScheduledThreadPool(1);
        ScheduledFuture scheduledFuture = scheduledExecutorService.schedule(new Callable() {
            public Object call() throws Exception {
                synchronized(stick_WaitCommand)
                {
                    command_timeout = true;
                    stick_WaitCommand.notify();
                }
                return "Called!";
            }
        },500, TimeUnit.MILLISECONDS);
         
        do 
        {
            synchronized(stick_WaitCommand)
            {
                stick_WaitCommand.wait();
            }
            //Thread.sleep(1);
            
            try
            {
                Comm.mutex.acquire();
                CMD_Found = command_found;
                CMD_Timeout = command_timeout;
                Status = Comm.SM;
            } finally {
                Comm.mutex.release();
            }
        }
        while ((CMD_Found == false) &&
                (CMD_Timeout == false) &&
                (Status != GUI.Response_SM.SM_ERROR));
        
        scheduledExecutorService.shutdownNow();
        try
            {
                Comm.mutex.acquire();
                Comm.WaitCmd = false;
            } finally {
                Comm.mutex.release();
            }
        
        if (CMD_Found == true)
        {
            GUI.Icon_Status.setIcon(new ImageIcon(getClass().getResource("/Resources/green.png")));
        }

        return CMD_Found;
    }
    
    
}
