/* --COPYRIGHT--,BSD
 * Copyright (c) 2014, Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * --/COPYRIGHT--*/
/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package SMBUsDemo;

import javax.swing.ImageIcon;

/**
 *
 * @author a0272600
 */
/*!
* \class TMP006_Comm
*
* \brief Class used to handle TMP006 Slave
*
* The TMP006 slave is a slave temperature sensor. This class has all functions needed to 
*  send commands to the Master device in order to read/write registers of the 
*  TMP006. Responses from TMP006->Master->PC will be reflected in the corresponding
*  GUI components. 
*  
*/
public class TMP006_Comm extends Slave_Comm{
    CommThread commthread;
    private static double Current_TMP006_AmbTemp_C =0 ;
    private static double Current_TMP006_Voltage_nV = 0;
    private static double Current_TMP006_ObjTemp = 0;

    private static final char READ_TMP006_VOLTAGE = 'v';
    private static final char READ_TMP006_AMBIENTTEMP = 't';
    private static final char READ_TMP006_MANUF_ID = 'm';
    private static final char READ_TMP006_DEVICE_ID = 'd';
    private static final char READ_TMP006_CONFIG = 'c';
    private static final char WRITE_TMP006_CONFIG = 'C';
    
    public static final char RESPONSE_TMP006_V = 0x03;
    public static final char RESPONSE_TMP006_T = 0x04;
    public static final char RESPONSE_TMP006_CONFIG = 0x05;
    public static final char RESPONSE_TMP006_MANID = 0x06;
    public static final char RESPONSE_TMP006_DEVID = 0x07;


    public TMP006_Comm() {
            
         commthread = new CommThread(); 
         commthread.start();
     }
    
    @Override
     public synchronized void Disable() {
         this.is_enabled = false;
         this.is_connected = false;
        GUI.Disable_TMP006_GUIComponents();
        GUI.Icon_TMP006.setIcon(new ImageIcon(getClass().getResource("/Resources/red.png")));
     }
     
     @Override
     public Boolean ProcessResponse()
    {
        String CmdResp;
        byte[] CmdRespByte;
        GUI.CommInterface Comm = GUI.MyCommSM;
                
        switch (Comm.RX_Cmd)
        {
            case RESPONSE_TMP006_MANID:
                CmdResp  = this.Get_ResponseAsString();
                if (CmdResp != null)
                {
                    GUI.Text_TMP006_MANUFID.setText(CmdResp); 
                    if (Comm.TX_Cmd ==  READ_TMP006_MANUF_ID)
                    {
                        synchronized(stick_WaitCommand)
                        {
                            command_found = true;
                            stick_WaitCommand.notify();
                        }
                    }

                    Comm.SM = GUI.Response_SM.SM_DONE;  // Packet reception is complete
                }
            break;
                
            case RESPONSE_TMP006_V:
                CmdResp  = this.Get_ResponseAsString();
                CmdRespByte  = this.Get_ResponseAsArray();
                if (CmdResp != null)
                {
                    GUI.Text_TMP006_VHex.setText(CmdResp);  
                    if (Comm.TX_Cmd ==  READ_TMP006_VOLTAGE)
                    {
                        synchronized(stick_WaitCommand)
                        {
                            command_found = true;
                            stick_WaitCommand.notify();
                        }
                    }

                     Current_TMP006_Voltage_nV = ((byte)CmdRespByte[0] + ((byte)CmdRespByte[1]<<8));
                     Current_TMP006_Voltage_nV *= 156.25;
                     if (Math.abs(Current_TMP006_Voltage_nV) > 1000000)
                     {
                         GUI.Text_TMP006_Volt.setText(String.format("%.2fmV", Current_TMP006_Voltage_nV/1000000));   
                     }
                     else if (Math.abs(Current_TMP006_Voltage_nV) > 1000)
                     {
                        GUI.Text_TMP006_Volt.setText(String.format("%.2fuV", Current_TMP006_Voltage_nV/1000)); 
                     }
                     else
                     {
                        GUI.Text_TMP006_Volt.setText(String.format("%.2fnV", Current_TMP006_Voltage_nV));  
                     }

                     // Calculate object temperature if we have the ambient temp
                     if (Current_TMP006_AmbTemp_C != 0)
                     {
                         final double a1 = 1.75E-3;
                         final double a2 = -1.677E-5;
                         final double b0 = -2.94E-5;
                         final double b1 = -5.7E-7;
                         final double b2 = 4.63E-9;
                         final double c2 = 13.4;
                         final double Tref = 298.15;
                         final double S0 = 6E-14;
                         double Tdie = Current_TMP006_AmbTemp_C + 273.15; // From C To Kelvin
                         double Vobj = Current_TMP006_Voltage_nV * 1E-9; // From nV to V

                         //S = S0[1 + a1(Tdie-Tref) + a2(Tdie-Tref)^2]
                         double S = S0 * ( 1 + a1 *(Tdie-Tref) + 
                                           a2 *(Math.pow(Tdie-Tref,2)) );
                         // Vos = b0 +b1(Tdie-Tref) +b2(Tdie-Tref)^2
                         double Vos = b0 + b1 * (Tdie - Tref) + 
                                    b2 * (Math.pow(Tdie - Tref,2));
                         // FVobj = (Vobj-Vos) + c2(Vobj-Vos)^2
                         double FVobj = (Vobj-Vos) + c2* (Math.pow(Vobj - Vos,2));
                         // Tobj = (Tdie^4 + (FVobj/S))^1/4
                         Current_TMP006_ObjTemp = Math.pow(Math.pow(Tdie, 4) +
                                                            (FVobj/S) ,0.25);
                         Current_TMP006_ObjTemp -= 273.15;
                         GUI.ObjectTherm.Data.setValue(Current_TMP006_ObjTemp);
                     }

                     Comm.SM = GUI.Response_SM.SM_DONE;  // Packet reception is complete
                } // if (CmdResp != null)
            break;
            
            case RESPONSE_TMP006_T:
                CmdResp  = this.Get_ResponseAsString();
                CmdRespByte  = this.Get_ResponseAsArray();
                if ((CmdResp != null) && (CmdRespByte != null))
                {
                    GUI.Text_TMP006_THex.setText(CmdResp);  
                    if (Comm.TX_Cmd ==  READ_TMP006_AMBIENTTEMP)
                    {
                        synchronized(stick_WaitCommand)
                        {
                            command_found = true;
                            stick_WaitCommand.notify();
                        }
                    }

                    Current_TMP006_AmbTemp_C = ((byte)CmdRespByte[0] + ((byte)CmdRespByte[1]<<8))>>2;
                    Current_TMP006_AmbTemp_C /= 32;
                    GUI.Text_TMP006_TC.setText(String.format("%.1f°C", Current_TMP006_AmbTemp_C)); 
                    GUI.AmbientTherm.Data.setValue(Current_TMP006_AmbTemp_C);
                    Comm.SM = GUI.Response_SM.SM_DONE;  // Packet reception is complete
                }
            break;
            
            case RESPONSE_TMP006_DEVID:
                CmdResp  = this.Get_ResponseAsString();
                if (CmdResp != null)
                {
                    GUI.Text_TMP006_DEVICEID.setText(CmdResp); 
                    if (Comm.TX_Cmd ==  READ_TMP006_DEVICE_ID)
                    {
                        synchronized(stick_WaitCommand)
                        {
                            command_found = true;
                            stick_WaitCommand.notify();
                        }
                    }

                    Comm.SM = GUI.Response_SM.SM_DONE;  // Packet reception is complete
                }
            break;
        
            case RESPONSE_TMP006_CONFIG:
               CmdRespByte  = this.Get_ResponseAsArray();
                if (CmdRespByte != null)
                {
                    // Check The conversion rate
                    if ( ((CmdRespByte[1] & 0x0E)>>1) > 4)
                    {
                        // Unexpected value for Configuration of Conv/rate
                        Comm.SM = GUI.Response_SM.SM_ERROR;
                        break;
                    }
                    else
                    {
                        // Set conversion rate
                        GUI.jComboBox_TMP006_ConvRate.setSelectedIndex((CmdRespByte[1] & 0x0E)>>1);   
                    }
                    // Check the mode, expected values are 0x00 or 0x70
                    if ( ((CmdRespByte[1] & 0x70) != 0x00) && 
                            ((CmdRespByte[1] & 0x70) != 0x70))
                    {
                        // Unexpected value for Configuration of Mode
                        Comm.SM = GUI.Response_SM.SM_ERROR;
                        break;
                    }else{
                        if ((CmdRespByte[1] & 0x70) == 0x00)
                        {
                            GUI.jComboBox_TMP006_MOD.setSelectedIndex(0); 
                        }
                        else
                        {
                            GUI.jComboBox_TMP006_MOD.setSelectedIndex(1); 
                        }
                    }

                    // Check DRDY bit
                    if ((CmdRespByte[1] & 0x01) == 0x00)
                    {
                        GUI.jCheckBox_TMP006_DRDY.setSelected(false); 
                    }
                    else
                    {
                        GUI.jCheckBox_TMP006_DRDY.setSelected(true); 
                    }

                    // Update the state machine
                    if (Comm.TX_Cmd ==  READ_TMP006_CONFIG)
                    {
                        synchronized(stick_WaitCommand)
                        {
                            command_found = true;
                            stick_WaitCommand.notify();
                        }
                    }

                    Comm.SM = GUI.Response_SM.SM_DONE;  // Packet reception is complete
                }
            break;
        }
        
        return true;
    }
     
     public void Change_Config(){
         byte[] data = new byte[] {
            0x00,                   // LSB
            0x00};                  // MSB
        
         
        // Change configuration of TMP006
        // Set the Mode of operation in bits [14:12]
         if (GUI.jComboBox_TMP006_MOD.getSelectedIndex() == 1)
             data[1] |= 0x70;
        
        // Set the Conversion Rate (0-4) in bits [11:9]
        data[1] |= GUI.jComboBox_TMP006_ConvRate.getSelectedIndex() << 1;
        
        // Set the EN bit
        if (GUI.jCheckBox_TMP006_DRDY.isSelected() == true)
            data[1] |= 0x01;

        Write_Config(data);
     }
    
    public void Read_AmbientTemp()
    {
        byte[] command = new byte[] {
            (byte)0x80,     // Header
            0x01,           // Len
            READ_TMP006_AMBIENTTEMP};     // Command

        SendCommand(command);
    }
    
    public void Read_Voltage()
    {
        byte[] command = new byte[] {
            (byte)0x80,     // Header
            0x01,           // Len
            READ_TMP006_VOLTAGE};     // Command

        SendCommand(command);
    }
    
    public void Read_ManufID()
    {
        byte[] command = new byte[] {
            (byte)0x80,     // Header
            0x01,           // Len
            READ_TMP006_MANUF_ID};     // Command

        SendCommand(command);
    }
    
    public Boolean Read_ManufID_WaitUntilDone()
    {
        byte[] command = new byte[] {
            (byte)0x80,     // Header
            0x01,           // Len
            READ_TMP006_MANUF_ID};     // Command

        try {
            return SendCommand_WaitUntilDone(command);
        }
        catch (InterruptedException e1){
                e1.printStackTrace();
                return false;
            }
    }
    
    public void Read_DeviceID()
    {
        byte[] command = new byte[] {
            (byte)0x80,     // Header
            0x01,           // Len
            READ_TMP006_DEVICE_ID};     // Command

        SendCommand(command);
    }
    
    public Boolean Read_DeviceID_WaitUntilDone()
    {
        byte[] command = new byte[] {
            (byte)0x80,     // Header
            0x01,           // Len
            READ_TMP006_DEVICE_ID};     // Command

        try {
            return SendCommand_WaitUntilDone(command);
        }
        catch (InterruptedException e1){
                e1.printStackTrace();
                return false;
            }
    }
    
    public void Read_Config()
    {
        byte[] command = new byte[] {
            (byte)0x80,     // Header
            0x01,           // Len
            READ_TMP006_CONFIG};     // Command

        SendCommand(command);
    }
    
    public Boolean Read_Config_WaitUntilDone()
    {
        byte[] command = new byte[] {
            (byte)0x80,     // Header
            0x01,           // Len
            READ_TMP006_CONFIG};     // Command

        try {
            return SendCommand_WaitUntilDone(command);
        }
        catch (InterruptedException e1){
                e1.printStackTrace();
                return false;
            }
    }
    
    public void Write_Config( byte[] data)
    {
        byte[] command = new byte[] {
            (byte)0x80,     // Header
            0x03,           // Len
            WRITE_TMP006_CONFIG,    // Command
            data[0],           // Data L
            data[1]};          // Data H

        SendCommand(command);
    }
    
    
    public class CommThread extends Thread {
    public void run(){
        try {
            while (true) {
                while (is_enabled == false)
                {
                    synchronized (stick) {
                        stick.wait();
                      }
                }
                if (is_enabled == true)
                {
                    //  If not connected yet, try to connect for first time
                    if (is_connected == false)
                    {
                        is_connected = true; // enable for now
                        if ((Read_DeviceID_WaitUntilDone() == true) && 
                            (Read_ManufID_WaitUntilDone() == true) &&
                            (Read_Config_WaitUntilDone() == true))
                        {
                            GUI.Enable_TMP006_GUIComponents();
                            is_connected = true;
                            GUI.Icon_TMP006.setIcon(new ImageIcon(getClass().getResource("/Resources/green.png")));
                        }
                        else
                        {
                            GUI.Disable_TMP006_GUIComponents();
                            GUI.Icon_TMP006.setIcon(new ImageIcon(getClass().getResource("/Resources/red.png")));
                            is_connected = false;
                        }
                           CommThread.sleep(500);
                    }
                    else
                    {
                        // If already connected yet, check if communication is still OK periodically
                        if (Read_DeviceID_WaitUntilDone() == true)
                        {
                            is_connected = true;
                            GUI.Enable_TMP006_GUIComponents();
                            GUI.Icon_TMP006.setIcon(new ImageIcon(getClass().getResource("/Resources/green.png")));
                        }
                        else
                        {
                            GUI.Disable_TMP006_GUIComponents();
                            GUI.Icon_TMP006.setIcon(new ImageIcon(getClass().getResource("/Resources/red.png")));
                            is_connected = false;
                        }
                        CommThread.sleep(2000);
                    }
                }
            }
        } catch (InterruptedException e) {
            e.printStackTrace();
        }
    }
  }
}
