/* --COPYRIGHT--,BSD
 * Copyright (c) 2014, Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * --/COPYRIGHT--*/
#ifndef __DIM430_H__
#define __DIM430_H__

//
// Include files
//
#include <stdint.h>
#include "msp430.h"
#include "smbus.h"

//
// Macros and Definitions
//
#define DIM430_SLAVE_ADDRESS         (0x43) /*! DIM430 Slave address */
#define DIM430_DEVICE_ID             (0x43) /*! Expected Device ID */

#define DIM430_CONFIG_REG            (0x00) /*! Address of CONFIG Register*/
#define DIM430_DUTY_REG              (0x01) /*! Address of DUTY_CYCLE reg */
#define DIM430_DEV_ID_REG            (0xFF) /*! Address of DEVICE_ID reg  */

/*! Software timeout waiting for a response */
#define DIM430_TIMEOUT                100000   
   
/*! Macro used to write CONFIG Register */
#define DIM430_SET_CONFIG(x,y)  DIM430_WriteReg_wTimeout(x, DIM430_CONFIG_REG,\
                                                       y, DIM430_TIMEOUT)
/*! Macro used to Read CONFIG Register */
#define DIM430_READ_CONFIG(x,y) DIM430_ReadReg_wTimeout(x, DIM430_CONFIG_REG,\
                                                       y, DIM430_TIMEOUT)
/*! Macro used to write DUTY CYCLE Register */
#define DIM430_SET_DUTY(x,y)    DIM430_WriteReg_wTimeout(x, DIM430_DUTY_REG,\
                                                       y, DIM430_TIMEOUT)
/*! Macro used to Read DUTY CYCLE Register */
#define DIM430_READ_DUTY(x,y)   DIM430_ReadReg_wTimeout(x, DIM430_DUTY_REG,\
                                                       y, DIM430_TIMEOUT)
/*! Macro used to Read DEVICE ID Register */
#define DIM430_READ_DEVID(x,y)  DIM430_ReadReg_wTimeout(x, DIM430_DEV_ID_REG,\
                                                       y, DIM430_TIMEOUT)
   
//
// Function prototypes
//
extern int8_t DIM430_Init( SMBus *SMB );
extern int8_t DIM430_ReadReg_wTimeout(SMBus *SMB, 
                                 uint8_t reg, 
                                 uint16_t *data, 
                                 uint32_t timeout);
extern int8_t DIM430_WriteReg_wTimeout(SMBus *SMB, 
                                 uint8_t reg, 
                                 uint16_t *data, 
                                 uint32_t timeout);


#endif //__DIM430_H__
