/* --COPYRIGHT--,BSD
 * Copyright (c) 2014, Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * --/COPYRIGHT--*/
#ifndef __TI_SERIALCOMM_H__
#define __TI_SERIALCOMM_H__

//
// Include files
//
#include <stdint.h>
#include <stdbool.h>


//
// Macros and definitions
//
/*! Expected packet header */
#define SERIALCOMM_HEADER           0x80    
/*! Response if no error is found */
#define SERIALCOMM_NO_ERROR         0x00    
/*! Response if invalid header was detected */
#define SERIALCOMM_INVALID_HDR      0x31    
/*! Response if the lenght of the packet is invalid */
#define SERIALCOMM_INVALID_LEN      0x32

//
// Type definitions
//
/*! Handler called when a command matches one in our list */
typedef void (*Error_Function_t)(uint8_t Error);

/*! State machine of Serial Communication */
typedef enum
{
    SerialComm_Idle=0,              /*! SerialComm is idle          */
    SerialComm_WaitPacketHDR,       /*! SerialComm is waiting for Packet header */
    SerialComm_WaitLen,             /*! SerialComm is waiting for Len */
    SerialComm_RxData,              /*! SerialComm is receiving data  */
    SerialComm_Error,               /*! SerialComm error */
    SerialComm_CMDReceived,         /*! Command was received */
}t_SerialComm_State;



//
// Functions 
// 
extern void TI_SerialComm_Init(uint32_t busClk);
extern void TI_SerialComm_SetErrorHandler(Error_Function_t ErrorHandler);
extern void TI_SerialComm_StartPacketReception(uint8_t *RXBuf, uint8_t RXLen);
extern t_SerialComm_State TI_SerialComm_GetState(void);
extern uint8_t TI_SerialComm_GetRxAvail(void);
extern bool TI_SerialComm_PacketComplete(void);
extern void TI_SerialComm_TxByte(uint8_t Data);
extern void TI_SerialComm_TxString(uint8_t *Data);
extern void TI_SerialComm_TxHexByte(uint8_t Data);
extern void TI_SerialComm_TxArray(uint8_t *Data, uint16_t len);
extern void TI_SerialComm_TxPacket(uint8_t response, 
                            uint8_t ctrl, 
                            uint8_t status,
                            uint8_t Txlen,
                            uint8_t TxAddr, 
                            uint8_t TxCmd, 
                            uint8_t *TxData , 
                            uint8_t *RespData , 
                            uint8_t Resplen);

#endif //__TI_SERIALCOMM_H__
