/* --COPYRIGHT--,BSD
 * Copyright (c) 2014, Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * --/COPYRIGHT--*/
#ifndef __TMP006_H__
#define __TMP006_H__

//
// Include files
//
#include <stdint.h>
#include "msp430.h"
#include "smbus.h"

//
// Macros and Definitions
//
#define TMP006_SLAVE_ADDRESS    (0x40)   /*! TMP006 Slave address    */
#define TMP006_DEVICE_ID        (0x0067) /*! Expected Device ID      */  
#define TMP006_MAN_ID           (0x5449) /*! Expected Manufacture ID */
                                                 
#define TMP006_SENS_V_REG       (0x00)   /*! Address of Sensor_V register     */
#define TMP006_TEMP_AMB_REG     (0x01)   /*! Address of Temperature register  */
#define TMP006_CONFIG_REG       (0x02)   /*! Address of Config Register       */
#define TMP006_MANUF_ID_REG     (0xFE)   /*! Address of Manufacturer ID Reg   */
#define TMP006_DEV_ID_REG       (0xFF)   /*! Address of Device ID register    */

#define TMP006_TIMEOUT          100000   /*! Software timeout  */
   
/*! Macro used to write Config Register */
#define TMP006_SET_CONFIG(x,y)  TMP006_WriteReg_wTimeout(x, TMP006_CONFIG_REG,\
                                                       y, TMP006_TIMEOUT)
/*! Macro used to read Config register */
#define TMP006_READ_CONFIG(x,y) TMP006_ReadReg_wTimeout(x, TMP006_CONFIG_REG,\
                                                       y, TMP006_TIMEOUT)
/*! Macro used to read Manufacturer ID Register */
#define TMP006_READ_MANID(x,y)  TMP006_ReadReg_wTimeout(x, TMP006_MANUF_ID_REG,\
                                                       y, TMP006_TIMEOUT)
/*! Macro used to Read Device ID Register */
#define TMP006_READ_DEVID(x,y)  TMP006_ReadReg_wTimeout(x, TMP006_DEV_ID_REG,\
                                                       y, TMP006_TIMEOUT)
/*! Macro used to read Sensor Voltage Register */
#define TMP006_READ_SENS_V(x,y) TMP006_ReadReg_wTimeout(x, TMP006_SENS_V_REG,\
                                                       y, TMP006_TIMEOUT)
/*! Macro used to read Ambient temperature Register */
#define TMP006_READ_TEMP_AMB(x,y)   TMP006_ReadReg_wTimeout(x, TMP006_TEMP_AMB_REG,\
                                                       y, TMP006_TIMEOUT)

//
// Function prototypes
//
extern int8_t TMP006_Init( SMBus *SMB );
extern int8_t TMP006_ReadReg_wTimeout(SMBus *SMB, 
                                 uint8_t reg, 
                                 uint16_t *data, 
                                 uint32_t timeout);
extern int8_t TMP006_WriteReg_wTimeout(SMBus *SMB, 
                                 uint8_t reg, 
                                 uint16_t *data, 
                                 uint32_t timeout);


#endif //__TMP006_H__
